//----------------------------------------------------------------------------
// 
//	file: simenv.h
// 
//	function: controls all simulation specific environment. 
//		1. register custom commands
//		2. register custom variables
//		3. list of commands with parameters
//		4. run custom commands
//		5. help 
//		6. dependencies
//		7. hooks
//		
//	author:   Yoonsuck Choe <yschoe@cs.utexas.edu>
//	creation: Fri Oct  9 20:50:33 CDT 1998
// 
//	$Revision: 1.3 $ $Date: 1999/02/09 07:42:25 $
//	$Log: simenv.h,v $
//	Revision 1.3  1999/02/09 07:42:25  yschoe
//	1. alias setting allowed
//	2. more stuff printed when status is asked
//
//	Revision 1.2  1998/10/26 03:54:11  yschoe
//	1. added hook counts for different kinds of hooks
//	2. added hook tables for the same
//	3. check for command redundancy when adding command
//	4. implemented all supported actions
//	5. added parse_file() for reading and executing commands from a file
//	   directly
//	6. added list_hooks() for a list of hooks
//	7. ** added rerun_hooks() : to be used as a preparation stage for
//	   loading a network image (in netlayer class)
//
//	Revision 1.1  1998/10/16 10:08:04  yschoe
//	Initial revision
//
//
//----------------------------------------------------------------------------
#ifndef __SIMENV_H
#define __SIMENV_H

#include "parser.h"
#include "blackboard.h"
#include "netlayer.h"

// Define some constants
#define SIM_STR_LEN 		BB_NAME_LEN
#define SIM_ARG_LEN 		40
#define SIM_MAX_CMD_ARGS	20

// Call member function of a certain object
// 	e.g. CALL_MEMBER_FUNC(map,netlayer::activate)(arg1,arg2,...)
//
#define CALL_FUNC(ptrToFunc) ((ptrToFunc))
#define CALL_MEMBER_FUNC(obj,ptrToMember) ((obj)->*(ptrToMember))

// default interface for all commands
typedef int (*SIM_FUNCP)(int argc, char *argv[]);
typedef int (netlayer::*SIM_MEMBER_FUNCP)(int argc, char *argv[]);
typedef netlayer* SIM_OBJP; // todo: this is too much dependent on

// hook exec states
enum SIM_HOOK_STAT {SIM_HOOK_WAITING, SIM_HOOK_EXECED, SIM_HOOK_ALL};

//
// 1. struct to hold object and command items
//
typedef struct {
 	char      name[BB_NAME_LEN];      // fixed sized name
	SIM_OBJP  objp;			// object pointer
} SIM_OBJ;

//
// 2. struct to hold command pointers
// 
typedef struct {
	char      	  name[BB_NAME_LEN];      // fixed sized name
  	SIM_OBJP   	  objp;
	SIM_FUNCP 	  funcp;
	SIM_MEMBER_FUNCP  member_funcp;
	int	   	  num_args;
	char	   	  help_str[SIM_STR_LEN];
} SIM_CMD;

//
// 3. struct to hold command hooks
//
typedef struct {
	char		command[SIM_STR_LEN];
	int		time;
	SIM_HOOK_STAT 	exec_flag;
} SIM_HOOK;	

//----------------------------------------------------------------------------
//
// The simenv class
//
//----------------------------------------------------------------------------
class simenv : public blackboard {

  int	  	obj_count;
  int	  	cmd_count;
  int		hook_init_count;
  int		hook_after_count;
  int		hook_before_count;
  char		*sim_argv[SIM_MAX_CMD_ARGS];
  parser 	*sim_parser;
  SIM_OBJ	*obj_table;
  SIM_CMD	*cmd_table;
  SIM_HOOK 	*hook_init_table;
  SIM_HOOK 	*hook_after_table;
  SIM_HOOK 	*hook_before_table;

public:


  // 1. Constructor and destructor  : do nothing much
  simenv(char *comment, int max);
  ~simenv();
 
  // 2. Extend blackboard to enable registering functions and parameters

  void 	add_obj(char *name, netlayer *obj, char *comment);
  void	add_alias(char *alias, char *original);
  void  add_cmd(char *name, SIM_FUNCP function, 
	int num_args, char *help_string);
  void  add_cmd(char *name, char *obj_name, 
		SIM_MEMBER_FUNCP function, int num_args, char *help_string);
  // void add_parameter () : reuse 

  // 3. Once all necessary object/var/command have been registered,
  //    read in actual commands from file and setup a scheduler. 
  //
  //	command syntax: given as a single character string
  //
  //	  3.1 global parameters 
  //		set <parameter> = <value>
  //		set <parameter>[<column>] = <value>
  //		set <parameter>[<col>][<row>] = <value>
  //	  3.2 parameter specific to an object
  //		obj <obj> set <parameter> = <value>
  //		obj <obj> set <parameter>[<column>] = <value>
  //		obj <obj> set <parameter>[<col>][<row>] = <value>
  //	  3.3 actions to be taken at certain time t
  //		hook init  <commands in 3.1 or 3.2> 
  //		hook before <exact-time> <commands in 3.1 or 3.2>
  //		hook after <exact-time> <commands in 3.1 or 3.2>
  //	  3.4 run command now
  //		run <registered-command> <args>
  //
  void  parse(char *command);
  int   parse_file(char *filename); // read lines from file and call parse()
protected:
  void  set_objparam();	// 
  void  set_hook();	// --> use sim_parser for processing the commands
  void  run();		// 

public:
  // 4. Check and run commands for iteration 't'
  void  run_hooks_init();
  void  run_hooks_after(int t);
  void  run_hooks_before(int t);
  void  list_hook(SIM_HOOK_STAT stat);
  void  rerun_hooks(int t);

  // 5. Show status of the environment (and show help)
  void  list_obj();
  void  list_cmd();
  void  list_param();
  void  help();	// list all command helps

  // 6. Override previous definition in blackboard class.
  int   save(char *filename);   // save blackboard 
  int   load(char *filename);   // load blackboard 

  int   find_param(char *name);  // find a certain named variable and return idx
  int   find_obj(char *name);    // find a certain named object and return idx
  int   find_cmd(char *name);    // find a certain named commnd and return idx

  void  list(); // list variable names & (added) list of objects and commands

private:
 
  // privately used initialization routines 
  void init_obj(char *name, SIM_OBJ *obj);
  void init_cmd(char *name, SIM_CMD *cmd, int num_args,
        char *help_str); 

  inline void chop (char *str) { str[strlen(str)-1] = '\0'; }

};

#endif

