## usage: [E,V,Emax,Vmax,Eimg] = convedgematnew(Img, kernelsize, gridsize, freq)
##
##	Img: p x q matrix (grayscale image)
##
##		Img: 2D matrix with grayscale image
##		kernelsize: width (==height) of convolution kernel
##		gridsize:   sampling grid size
##		freq: 	    gabor frequency (relative to the kernelsize) 
##
## Returns a n x m matrix E where 
##	n = p / gridsize
##	m = q / gridsize
##	and each E_ij is the edge orientation of the subimage patch.
##
## First, convolve with DoG filter, and then convolve with gabor
## filters of orientations 0, 60, 120. Do a polar addition. Cut the generated
## images into patches and find the mximum value and the index	
##
##	Usage example:
##		[E,V,Emax,Vmax,Eimg] = convedgematnew(imagemat,9,3,1);
##
##		both DoG and gabor filters are  9 x 9, and
##		Gabor frequency is 1.
##		final grid sampling size is 3 x 3.
##
## $Id: convedgemat.m,v 1.2 2002/03/09 17:31:27 yschoe Exp yschoe $

function [E,V,Emax,Vmax,Eimg] = convedgematnew(Img, kernelsize, gridsize, freq)

	# get image size
	[oldp, oldq] = size(Img);

	# adjust grid size to have the conv kernels
	# centered at the middle: i.e. make the gridsize an odd number.
	if ( rem(kernelsize,2) == 0 ) 
		kernelsize ++;
	endif

	DoG = dog(kernelsize);

        # convolve with DoG, and only use the abs part
	Eimg = real(conv2(Img,DoG,"same"));	

	    
        # calculate new image size to make it divisible by gridsize
	p = floor(oldp/gridsize)*gridsize;
	q = floor(oldq/gridsize)*gridsize;

   	ygrids = floor(p/gridsize);
	xgrids = floor(q/gridsize);

	#initialize matrices
	V = zeros(ygrids,xgrids).-999999;
	E = zeros(ygrids,xgrids);
	
        Vmax=zeros(oldp,oldq);
	V1max=zeros(oldp,oldq);
        V2max=zeros(oldp,oldq);
        V3max=zeros(oldp,oldq);

        Emax=zeros(oldp,oldq);
       
	g1 = gabor(kernelsize,kernelsize,"sin",0,freq);
	g2 = gabor(kernelsize,kernelsize,"cos",0,freq);

        g3 = gabor(kernelsize,kernelsize,"sin",60,freq);
	g4 = gabor(kernelsize,kernelsize,"cos",60,freq);
  
        g5 = gabor(kernelsize,kernelsize,"sin",120,freq);
	g6 = gabor(kernelsize,kernelsize,"cos",120,freq);

        V1max=real(conv2(Eimg,g1,"same")).^2+real(conv2(Eimg,g2,"same")).^2;
        V2max=real(conv2(Eimg,g3,"same")).^2+real(conv2(Eimg,g4,"same")).^2;	       V3max=real(conv2(Eimg,g5,"same")).^2+real(conv2(Eimg,g6,"same")).^2;

        for i=1:oldp
	for j=1:oldq
	xval = (V1max(i,j)*cos(0))+(V2max(i,j)*cos(pi/3))+(V3max(i,j)*cos(2*pi/3));
        yval = (V1max(i,j)*sin(0))+(V2max(i,j)*sin(pi/3))+(V3max(i,j)*sin(2*pi/3));       
         Emax(i,j) = floor(atan(yval/xval)*(180/pi));
        if (Emax(i,j) < 0)
          Emax(i,j) = 180-abs(Emax(i,j)); 
        endif
        Emax(i,j) = round(Emax(i,j)/10)+1;
 
        Vmax(i,j) = sqrt((xval^2)+(yval^2));
        endfor
        endfor    

        #cut Vmax and Emax into grids and calculate V and E
 	for j=1:ygrids
          for i=1:xgrids
		vtmp = vec(Vmax(1+((j-1)*gridsize):(j*gridsize),
			        1+((i-1)*gridsize):(i*gridsize)));
		# idx contains the index of the max within that subimage
		# block.
  	        [V(j,i),idx]=max(vtmp);
		etmp = vec(Emax(1+((j-1)*gridsize):(j*gridsize),
				1+((i-1)*gridsize):(i*gridsize)));
                E(j,i)=etmp(idx);
          endfor
        endfor

endfunction
